//
//  SMGeometry.h
//  Drawings
//
//  Created by Stephan Michels on 25.02.12.
//  Copyright (c) 2012 Stephan Michels Softwareentwicklung und Beratung. All rights reserved.
//

#import <Foundation/Foundation.h>


// 64-bit float macros

#ifdef __LP64__
	#define CGFloatFabs( n )		fabs( n )
	#define CGFloatTrunc( n )		trunc( n )
	#define CGFloatRound( n )		round( n )
	#define CGFloatLround( n )		lround( n )
	#define CGFloatFloor( n )		floor( n )
	#define CGFloatCeil( n )		ceil( n )
	#define CGFloatExp( n )			exp( n )
	#define CGFloatSqrt( n )		sqrt( n )
	#define CGFloatLog( n )			log( n )
	#define CGFloatLog10( n )		log10( n )
	#define CGFloatSin( n )			sin( n )
	#define CGFloatCos( n )			cos( n )
	#define CGFloatTan2( n, m )		tan2( n, m )
	#define CGFloatAtan2( n, m )	atan2( n, m )
	#define CGFloatHypot( n, m )	hypot( n, m )
#else
	#define CGFloatFabs( n )		fabsf( n )
	#define CGFloatTrunc( n )		truncf( n )
	#define CGFloatRound( n )		roundf( n )
	#define CGFloatLround( n )		lround((double) n )
	#define CGFloatFloor( n )		floorf( n )
	#define CGFloatCeil( n )		ceilf( n )
	#define CGFloatExp( n )			expf( n )
	#define CGFloatSqrt( n )		sqrtf( n )
	#define CGFloatLog10( n )		log10f( n )
	#define CGFloatSin( n )			sinf( n )
	#define CGFloatCos( n )			cosf( n )
	#define CGFloatTan2( n, m )		tan2f( n, m )
	#define CGFloatAtan2( n, m )	atan2( n, m )
	#define CGFloatHypot( n, m )	hypotf( n, m )
#endif

CGFloat SMSignum(CGFloat value);

BOOL CGPointIsZero(CGPoint point);
CGFloat CGFloatClamp(CGFloat value, CGFloat min, CGFloat max);
CGFloat CGFloatRestrictToInterval(CGFloat value, CGFloat minValue, CGFloat maxValue);
NSInteger CGFloatFloorWithEpsilon(double value);

CGPoint CGPointAdd(CGPoint point1, CGPoint point2);
CGPoint CGPointSub(CGPoint point1, CGPoint point2);
CGFloat CGPointDistanceToPoint(CGPoint point1, CGPoint point2);
CGFloat CGPointLength(CGPoint point);
CGFloat CGPointLengthSquared(CGPoint point);
CGPoint CGPointNormalize(CGPoint point);
CGPoint CGPointInvert(CGPoint point);
CGPoint CGPointOrthogonal(CGPoint point);
CGPoint CGPointScale(CGPoint point, CGFloat scale);
CGPoint CGPointScaleToLength(CGPoint point, double newlength);
CGPoint CGPointScaleAdd(CGPoint point1, CGPoint point2, CGFloat scale);
CGPoint CGPointScaleAdd2(CGPoint point1, CGPoint point2, CGFloat scale2, CGPoint point3, CGFloat scale3);
CGPoint CGPointScaleAdd3(CGPoint point1, CGPoint point2, CGFloat scale2, CGPoint point3, CGFloat scale3, CGPoint point4, CGFloat scale4);
CGFloat CGPointDot(CGPoint point1, CGPoint point2);
CGPoint CGPointMid(CGPoint point1, CGPoint point2);

CGPoint CGRectClampPoint(CGRect rect, CGPoint point);

CGFloat CGPointGetAngle(CGPoint point);
double CGPointDiffAngle(double angle, CGPoint point1, CGPoint point2);
CGPoint CGPointAngle(CGFloat angle);
CGFloat CGAngleWithPointAndCenter(CGPoint point, CGPoint centerPoint);

double SMIntersection(CGPoint b1point1, CGPoint b1point2, CGPoint b2point1, CGPoint b2point2);

typedef NS_ENUM(NSUInteger, SMRectAnchor) {
    SMRectAnchorTopLeft,
    SMRectAnchorTopCenter,
    SMRectAnchorTopRight,
    SMRectAnchorMiddleLeft,
    SMRectAnchorMiddleCenter,
    SMRectAnchorMiddleRight,
    SMRectAnchorBottomLeft,
    SMRectAnchorBottomCenter,
    SMRectAnchorBottomRight
};


CGPoint CGRectGetOriginForAnchor(CGRect rect, SMRectAnchor anchor);
CGRect CGRectSetOriginForAnchor(CGRect rect, CGPoint origin, SMRectAnchor anchor, BOOL resize);
CGRect CGRectSetSizeForAnchor(CGRect rect, CGSize size, SMRectAnchor anchor);
CGRect CGRectSetOriginXForAnchor(CGRect rect, CGFloat originX, SMRectAnchor anchor, BOOL resize);
CGRect CGRectSetOriginYForAnchor(CGRect rect, CGFloat originY, SMRectAnchor anchor, BOOL resize);
CGRect CGRectSetSizeWidthForAnchor(CGRect rect, CGFloat sizeWidth, SMRectAnchor anchor);
CGRect CGRectSetSizeHeightForAnchor(CGRect rect, CGFloat sizeHeight, SMRectAnchor anchor);
CGRect CGRectFromPoints(CGPoint point1, CGPoint point2);